@@ -9,9 +9,14 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import print_function
 
 from . import dbapi
 from . import client
 from . import constants
+from . import exceptions
+from . import logging
 
-__version__ = '0.3.0'
+__version__ = '0.4.1'
