@@ -32,8 +32,10 @@ The main interface is :class:`PrestoQuery`: ::
     >> query =  PrestoQuery(request, sql)
     >> rows = list(query.execute())
 """
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import print_function
 
-import logging
 import os
 from requests_kerberos.exceptions import KerberosExchangeError
 from typing import Any, Dict, List, Optional, Text, Tuple, Union  # NOQA for mypy types
@@ -42,14 +44,15 @@ import requests
 
 from prestodb import constants
 from prestodb import exceptions
+import prestodb.logging
+from prestodb.transaction import NO_TRANSACTION
 import prestodb.redirect
 
 
 __all__ = ['PrestoQuery', 'PrestoRequest']
 
 
-logger = logging.getLogger(__name__)
-logger.setLevel(logging.INFO)
+logger = prestodb.logging.get_logger(__name__)
 
 
 MAX_ATTEMPTS = constants.DEFAULT_MAX_ATTEMPTS
@@ -71,6 +74,7 @@ class ClientSession(object):
         user,
         properties=None,
         headers=None,
+        transaction_id=None,
     ):
         self.catalog = catalog
         self.schema = schema
@@ -80,6 +84,7 @@ class ClientSession(object):
             properties = {}
         self._properties = properties
         self._headers = headers or {}
+        self.transaction_id = transaction_id
 
     @property
     def properties(self):
@@ -199,6 +204,7 @@ class PrestoRequest(object):
         schema=None,  # type: Text
         session_properties=None,  # type: Optional[Dict[Text, Any]]
         http_headers=None,  # type: Optional[Dict[Text, Text]]
+        transaction_id=NO_TRANSACTION,  # type: Optional[Text]
         http_scheme=constants.HTTP,  # type: Text
         auth=constants.DEFAULT_AUTH,  # type: Optional[Any]
         redirect_handler=prestodb.redirect.GatewayRedirectHandler(),
@@ -206,6 +212,7 @@ class PrestoRequest(object):
         request_timeout=constants.DEFAULT_REQUEST_TIMEOUT,  # type: Union[float, Tuple[float, float]]
         handle_retry=exceptions.RetryWithExponentialBackoff(),
     ):
+        # type: (...) -> None
         self._client_session = ClientSession(
             catalog,
             schema,
@@ -213,13 +220,14 @@ class PrestoRequest(object):
             user,
             session_properties,
             http_headers,
+            transaction_id,
         )
 
         self._host = host
         self._port = port
         self._next_uri = None  # type: Optional[Text]
         # mypy cannot follow module import
-        self._http_session = self.http.Session()
+        self._http_session = self.http.Session()  # type: ignore
         self._http_session.headers.update(self.http_headers)
         self._auth = auth
         if self._auth:
@@ -233,6 +241,14 @@ class PrestoRequest(object):
         self.max_attempts = max_attempts
         self._http_scheme = http_scheme
 
+    @property
+    def transaction_id(self):
+        return self._client_session.transaction_id
+
+    @transaction_id.setter
+    def transaction_id(self, value):
+        self._client_session.transaction_id = value
+
     @property
     def http_headers(self):
         # type: () -> Dict[Text, Text]
@@ -255,6 +271,9 @@ class PrestoRequest(object):
                 raise ValueError('cannot override reserved HTTP header {}'.format(key))
         headers.update(self._client_session.headers)
 
+        transaction_id = self._client_session.transaction_id
+        headers[constants.HEADER_TRANSACTION] = transaction_id
+
         return headers
 
     @property
@@ -264,7 +283,7 @@ class PrestoRequest(object):
 
     @max_attempts.setter
     def max_attempts(self, value):
-        # type: int -> None
+        # type: (int) -> None
         self._max_attempts = value
         if value == 1:  # No retry
             self._get = self._http_session.get
@@ -276,8 +295,8 @@ class PrestoRequest(object):
             self._handle_retry,
             exceptions=(
                 KerberosExchangeError,
-                PrestoRequest.http.ConnectionError,
-                PrestoRequest.http.Timeout,
+                PrestoRequest.http.ConnectionError,  # type: ignore
+                PrestoRequest.http.Timeout,  # type: ignore
             ),
             conditions=(
                 # need retry when there is no exception but the status code is 503
@@ -290,7 +309,7 @@ class PrestoRequest(object):
         self._delete = with_retry(self._http_session.delete)
 
     def get_url(self, path):
-        # type: Text -> Text
+        # type: (Text) -> Text
         return "{protocol}://{host}:{port}{path}".format(
             protocol=self._http_scheme,
             host=self._host,
@@ -375,9 +394,7 @@ class PrestoRequest(object):
         )
 
     def process(self, http_response):
-        # type: requests.Response -> PrestoStatus:
-        status_code = http_response.status_code
-        # mypy cannot follow module import
+        # type: (requests.Response) -> PrestoStatus
         if not http_response.ok:
             self.raise_response_error(http_response)
 
@@ -454,10 +471,11 @@ class PrestoQuery(object):
         request,  # type: PrestoRequest
         sql,  # type: Text
     ):
-        self.query_id = None
+        # type: (...) -> None
+        self.query_id = None  # type: Optional[Text]
 
-        self._stats = {}
-        self._columns = None
+        self._stats = {}  # type: Dict[Any, Any]
+        self._columns = None  # type: Optional[List[Text]]
 
         self._finished = False
         self._cancelled = False
@@ -513,8 +531,8 @@ class PrestoQuery(object):
         return status.rows
 
     def cancel(self):
+        # type: () -> None
         """Cancel the current query"""
-        # type: None -> None
         if self.is_finished():
             return
 
