@@ -18,15 +18,14 @@ defined in pep-0249.
 from __future__ import absolute_import
 from __future__ import division
 from __future__ import print_function
-from __future__ import unicode_literals
 
 import functools
-import logging
 import random
 import time
 
+import prestodb.logging
 
-logger = logging.getLogger(__name__)
+logger = prestodb.logging.get_logger(__name__)
 
 
 class HttpError(Exception):
